from datetime import datetime
from typing import List, Optional, Dict, Any

from janis_core import *
from janis_core.types.common_data_types import File, Float, Int, Boolean, String

Mmseqs_Cluster_V0_1_0 = CommandToolBuilder(tool="mmseqs_cluster", base_command=["mmseqs", "cluster"], inputs=[ToolInput(tag="in_seed_sub_mat", input_type=File(optional=True), prefix="--seed-sub-mat", doc=InputDocumentation(doc="Substitution matrix file for k-mer generation [nucl:nucleotide.out,aa:VTML80.out]")), ToolInput(tag="in_sensitivity_faster_fast", input_type=Float(optional=True), prefix="-s", doc=InputDocumentation(doc="Sensitivity: 1.0 faster; 4.0 fast; 7.5 sensitive [4.000]")), ToolInput(tag="in_kmer_length_set", input_type=Int(optional=True), prefix="-k", doc=InputDocumentation(doc="k-mer length (0: automatically set to optimum) [0]")), ToolInput(tag="in_k_score", input_type=Int(optional=True), prefix="--k-score", doc=InputDocumentation(doc="k-mer threshold for generating similar k-mer lists [2147483647]")), ToolInput(tag="in_alph_size", input_type=Int(optional=True), prefix="--alph-size", doc=InputDocumentation(doc="Alphabet size (range 2-21) [nucl:5,aa:21]")), ToolInput(tag="in_max_seqs", input_type=Int(optional=True), prefix="--max-seqs", doc=InputDocumentation(doc="Maximum results per query sequence allowed to pass the prefilter (affects sensitivity) [20]")), ToolInput(tag="in_split", input_type=Int(optional=True), prefix="--split", doc=InputDocumentation(doc="Split input into N equally distributed chunks. 0: set the best split automatically [0]")), ToolInput(tag="in_split_mode", input_type=Int(optional=True), prefix="--split-mode", doc=InputDocumentation(doc="0: split target db; 1: split query db; 2: auto, depending on main memory [2]")), ToolInput(tag="in_split_memory_limit", input_type=Int(optional=True), prefix="--split-memory-limit", doc=InputDocumentation(doc="Set max memory per split. E.g. 800B, 5K, 10M, 1G. Default (0) to all available system memory [0]")), ToolInput(tag="in_comp_bias_corr", input_type=Int(optional=True), prefix="--comp-bias-corr", doc=InputDocumentation(doc="Correct for locally biased amino acid composition (range 0-1) [1]")), ToolInput(tag="in_diag_score", input_type=Boolean(optional=True), prefix="--diag-score", doc=InputDocumentation(doc="Use ungapped diagonal scoring during prefilter [1]")), ToolInput(tag="in_exact_km_er_matching", input_type=Int(optional=True), prefix="--exact-kmer-matching", doc=InputDocumentation(doc="Extract only exact k-mers for matching (range 0-1) [0]")), ToolInput(tag="in_mask", input_type=Int(optional=True), prefix="--mask", doc=InputDocumentation(doc="Mask sequences in k-mer stage: 0: w/o low complexity masking, 1: with low complexity masking [1]")), ToolInput(tag="in_mask_lower_case", input_type=Int(optional=True), prefix="--mask-lower-case", doc=InputDocumentation(doc="Lowercase letters will be excluded from k-mer search 0: include region, 1: exclude region [0]")), ToolInput(tag="in_min_un_gapped_score", input_type=Int(optional=True), prefix="--min-ungapped-score", doc=InputDocumentation(doc="Accept only matches with ungapped alignment score above threshold [15]")), ToolInput(tag="in_add_self_matches", input_type=Boolean(optional=True), prefix="--add-self-matches", doc=InputDocumentation(doc="Artificially add entries of queries with themselves (for clustering) [0]")), ToolInput(tag="in_spaced_km_er_mode", input_type=Int(optional=True), prefix="--spaced-kmer-mode", doc=InputDocumentation(doc="0: use consecutive positions in k-mers; 1: use spaced k-mers [1]")), ToolInput(tag="in_spaced_km_er_pattern", input_type=String(optional=True), prefix="--spaced-kmer-pattern", doc=InputDocumentation(doc="User-specified spaced k-mer pattern []")), ToolInput(tag="in_local_tmp", input_type=File(optional=True), prefix="--local-tmp", doc=InputDocumentation(doc="Path where some of the temporary files will be created []")), ToolInput(tag="in_list_matches_fraction", input_type=Float(optional=True), prefix="-c", doc=InputDocumentation(doc="List matches above this fraction of aligned (covered) residues (see --cov-mode) [0.800]")), ToolInput(tag="in_cov_mode", input_type=Int(optional=True), prefix="--cov-mode", doc=InputDocumentation(doc="0: coverage of query and target\n1: coverage of target\n2: coverage of query\n3: target seq. length has to be at least x% of query length\n4: query seq. length has to be at least x% of target length\n5: short seq. needs to be at least x% of the other seq. length [0]")), ToolInput(tag="in_add_string_convert", input_type=Boolean(optional=True), prefix="-a", doc=InputDocumentation(doc="Add backtrace string (convert to alignments with mmseqs convertalis module) [0]")), ToolInput(tag="in_alignment_mode", input_type=Int(optional=True), prefix="--alignment-mode", doc=InputDocumentation(doc="How to compute the alignment:\n0: automatic\n1: only score and end_pos\n2: also start_pos and cov\n3: also seq.id\n4: only ungapped alignment [3]")), ToolInput(tag="in_alignment_output_mode", input_type=Int(optional=True), prefix="--alignment-output-mode", doc=InputDocumentation(doc="How to compute the alignment:\n0: automatic\n1: only score and end_pos\n2: also start_pos and cov\n3: also seq.id\n4: only ungapped alignment\n5: score only (output) cluster format [0]")), ToolInput(tag="in_wrapped_scoring", input_type=Boolean(optional=True), prefix="--wrapped-scoring", doc=InputDocumentation(doc="Double the (nucleotide) query sequence during the scoring process to allow wrapped diagonal scoring around end and start [0]")), ToolInput(tag="in_list_matches_evalue", input_type=Float(optional=True), prefix="-e", doc=InputDocumentation(doc="List matches below this E-value (range 0.0-inf) [1.000E-03]")), ToolInput(tag="in_min_seq_id", input_type=Float(optional=True), prefix="--min-seq-id", doc=InputDocumentation(doc="List matches above this sequence identity (for clustering) (range 0.0-1.0) [0.000]")), ToolInput(tag="in_min_aln_len", input_type=Int(optional=True), prefix="--min-aln-len", doc=InputDocumentation(doc="Minimum alignment length (range 0-INT_MAX) [0]")), ToolInput(tag="in_seq_id_mode", input_type=Int(optional=True), prefix="--seq-id-mode", doc=InputDocumentation(doc="0: alignment length 1: shorter, 2: longer sequence [0]")), ToolInput(tag="in_alt_ali", input_type=Int(optional=True), prefix="--alt-ali", doc=InputDocumentation(doc="Show up to this many alternative alignments [0]")), ToolInput(tag="in_max_rejected", input_type=Int(optional=True), prefix="--max-rejected", doc=InputDocumentation(doc="Maximum rejected alignments before alignment calculation for a query is stopped [2147483647]")), ToolInput(tag="in_max_accept", input_type=Int(optional=True), prefix="--max-accept", doc=InputDocumentation(doc="Maximum accepted alignments before alignment calculation for a query is stopped [2147483647]")), ToolInput(tag="in_score_bias", input_type=Float(optional=True), prefix="--score-bias", doc=InputDocumentation(doc="Score bias when computing SW alignment (in bits) [0.000]")), ToolInput(tag="in_realign", input_type=Boolean(optional=True), prefix="--realign", doc=InputDocumentation(doc="Compute more conservative, shorter alignments (scores and E-values not changed) [0]")), ToolInput(tag="in_realign_score_bias", input_type=Float(optional=True), prefix="--realign-score-bias", doc=InputDocumentation(doc="Additional bias when computing realignment [-0.200]")), ToolInput(tag="in_realign_max_seqs", input_type=Int(optional=True), prefix="--realign-max-seqs", doc=InputDocumentation(doc="Maximum number of results to return in realignment [2147483647]")), ToolInput(tag="in_gap_open", input_type=Int(optional=True), prefix="--gap-open", doc=InputDocumentation(doc="Gap open cost [nucl:5,aa:11]")), ToolInput(tag="in_gap_extend", input_type=Int(optional=True), prefix="--gap-extend", doc=InputDocumentation(doc="Gap extension cost [nucl:2,aa:1]")), ToolInput(tag="in_z_drop", input_type=Int(optional=True), prefix="--zdrop", doc=InputDocumentation(doc="Maximal allowed difference between score values before alignment is truncated  (nucleotide alignment only) [40]")), ToolInput(tag="in_cluster_mode", input_type=Int(optional=True), prefix="--cluster-mode", doc=InputDocumentation(doc="0: Set-Cover (greedy)\n1: Connected component (BLASTclust)\n2,3: Greedy clustering by sequence length (CDHIT) [0]")), ToolInput(tag="in_max_iterations", input_type=Int(optional=True), prefix="--max-iterations", doc=InputDocumentation(doc="Maximum depth of breadth first search in connected component clustering [1000]")), ToolInput(tag="in_similarity_type", input_type=Int(optional=True), prefix="--similarity-type", doc=InputDocumentation(doc="Type of score used for clustering. 1: alignment score 2: sequence identity [2]")), ToolInput(tag="in_single_step_clustering", input_type=Boolean(optional=True), prefix="--single-step-clustering", doc=InputDocumentation(doc="Switch from cascaded to simple clustering workflow [0]")), ToolInput(tag="in_cluster_steps", input_type=Int(optional=True), prefix="--cluster-steps", doc=InputDocumentation(doc="Cascaded clustering steps from 1 to -s [3]")), ToolInput(tag="in_cluster_reassign", input_type=Boolean(optional=True), prefix="--cluster-reassign", doc=InputDocumentation(doc="Cascaded clustering can cluster sequence that do not fulfill the clustering criteria.\nCluster reassignment corrects these errors [0]")), ToolInput(tag="in_km_er_per_seq", input_type=Int(optional=True), prefix="--kmer-per-seq", doc=InputDocumentation(doc="k-mers per sequence [21]")), ToolInput(tag="in_km_er_per_seq_scale", input_type=Int(optional=True), prefix="--kmer-per-seq-scale", doc=InputDocumentation(doc="Scale k-mer per sequence based on sequence length as kmer-per-seq val + scale x seqlen [nucl:0.200,aa:0.000]")), ToolInput(tag="in_adjust_km_er_len", input_type=Boolean(optional=True), prefix="--adjust-kmer-len", doc=InputDocumentation(doc="Adjust k-mer length based on specificity (only for nucleotides) [0]")), ToolInput(tag="in_hash_shift", input_type=Int(optional=True), prefix="--hash-shift", doc=InputDocumentation(doc="Shift k-mer hash initialization [67]")), ToolInput(tag="in_include_only_extendable", input_type=Boolean(optional=True), prefix="--include-only-extendable", doc=InputDocumentation(doc="Include only extendable [0]")), ToolInput(tag="in_ignore_multi_km_er", input_type=Boolean(optional=True), prefix="--ignore-multi-kmer", doc=InputDocumentation(doc="Skip k-mers occurring multiple times (>=2) [0]")), ToolInput(tag="in_pc_a", input_type=Float(optional=True), prefix="--pca", doc=InputDocumentation(doc="Pseudo count admixture strength [1.000]")), ToolInput(tag="in_pcb", input_type=Float(optional=True), prefix="--pcb", doc=InputDocumentation(doc="Pseudo counts: Neff at half of maximum admixture (range 0.0-inf) [1.500]")), ToolInput(tag="in_re_score_mode", input_type=Int(optional=True), prefix="--rescore-mode", doc=InputDocumentation(doc="Rescore diagonals with:\n0: Hamming distance\n1: local alignment (score only)\n2: local alignment\n3: global alignment\n4: longest alignment fulfilling window quality criterion [0]")), ToolInput(tag="in_sub_mat", input_type=File(optional=True), prefix="--sub-mat", doc=InputDocumentation(doc="Substitution matrix file [nucl:nucleotide.out,aa:blosum62.out]")), ToolInput(tag="in_max_seq_len", input_type=Int(optional=True), prefix="--max-seq-len", doc=InputDocumentation(doc="Maximum sequence length [65535]")), ToolInput(tag="in_db_load_mode", input_type=Int(optional=True), prefix="--db-load-mode", doc=InputDocumentation(doc="Database preload mode 0: auto, 1: fread, 2: mmap, 3: mmap+touch [0]")), ToolInput(tag="in_threads", input_type=Int(optional=True), prefix="--threads", doc=InputDocumentation(doc="Number of CPU-cores used (all by default) [2]")), ToolInput(tag="in_compressed", input_type=Int(optional=True), prefix="--compressed", doc=InputDocumentation(doc="Write compressed output [0]")), ToolInput(tag="in_verbosity_level_errors", input_type=Int(optional=True), prefix="-v", doc=InputDocumentation(doc="Verbosity level: 0: quiet, 1: +errors, 2: +warnings, 3: +info [3]")), ToolInput(tag="in_remove_tmp_files", input_type=Boolean(optional=True), prefix="--remove-tmp-files", doc=InputDocumentation(doc="Delete temporary files [0]")), ToolInput(tag="in_force_reuse", input_type=Boolean(optional=True), prefix="--force-reuse", doc=InputDocumentation(doc="Reuse tmp filse in tmp/latest folder ignoring parameters and version changes [0]")), ToolInput(tag="in_mpi_runner", input_type=Int(optional=True), prefix="--mpi-runner", doc=InputDocumentation(doc="Use MPI on compute cluster with this MPI command (e.g. 'mpirun -np 42') []")), ToolInput(tag="in_filter_hits", input_type=Boolean(optional=True), prefix="--filter-hits", doc=InputDocumentation(doc="Filter hits by seq.id. and coverage [0]")), ToolInput(tag="in_sort_results", input_type=Int(optional=True), prefix="--sort-results", doc=InputDocumentation(doc="Sort results: 0: no sorting, 1: sort by E-value (Alignment) or seq.id. (Hamming) [0]"))], outputs=[ToolOutput(tag="out_seed_sub_mat", output_type=File(optional=True), selector=InputSelector(input_to_select="in_seed_sub_mat", type_hint=File()), doc=OutputDocumentation(doc="Substitution matrix file for k-mer generation [nucl:nucleotide.out,aa:VTML80.out]")), ToolOutput(tag="out_sub_mat", output_type=File(optional=True), selector=InputSelector(input_to_select="in_sub_mat", type_hint=File()), doc=OutputDocumentation(doc="Substitution matrix file [nucl:nucleotide.out,aa:blosum62.out]"))], container="quay.io/biocontainers/mmseqs2:13.45111--h2d02072_0", version="v0.1.0")


if __name__ == "__main__":
    # or "cwl"
    Mmseqs_Cluster_V0_1_0().translate("wdl")

